<?php
namespace AustraliaPost\Extensions\Business\Endpoints;

use AustraliaPost\Extensions\Business\API\Wrapper\Price_Feature;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Item;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Request;
use AustraliaPost\Extensions\Business\API\Wrapper\Shipment_Price;
use AustraliaPost\Extensions\Business\API\Wrapper\Shipment_Price_Request;
use AustraliaPost\Extensions\Business\Helpers\Utilities;

class Calculate_Prices_With_Surcharges_Endpoint extends Abstract_Endpoint {


	public function callback( $data )
	{
		$order = $this->get_order($data);
		$from = $this->get_from_address();
		$to = $this->get_to_address($order);

		$priceRequest = new Price_Request();
		$priceRequest
			->setFromPostcode($from->getPostcode())
			->setFromCountry($from->getCountry())
			->setToPostcode($to->getPostcode())
			->setToCountry($to->getCountry());

		if (!isset($data['shipments']) || !is_array($data['shipments'])) {
			$this->abort('shipments must be provided');
		}

		foreach ($data['shipments'] as $index => $shipment)
		{
			$priceItem = (new Price_Item(Utilities::accountType()))
				->setLength($shipment['length'] / 10)
				->setWidth($shipment['width'] / 10)
				->setHeight($shipment['height'] / 10)
				->setWeight($shipment['weight'] / 1000)
				->setProductId($shipment['product_id'])
				->setItemReference($order->get_id() .'#' . $index);

			if (isset($data['has_dangerous_goods']) && $data['has_dangerous_goods'] === true) {
				$priceItem->setContainsDangerousGoods(true);
			}

			$priceItem = $this->addProductIds($priceItem, $shipment);

			$priceItem = $this->addSignatureOnDelivery($priceItem, $shipment);


			if ($shipment['extra_cover'] > 0) {
				$extra_cover_feature = new Price_Feature();
				$extra_cover_feature->setCode('TRANSIT_COVER');
				$extra_cover_feature->setAttributes(['cover_amount' => round(floatval($shipment['extra_cover']), 2)]);
				$priceItem->addFeature($extra_cover_feature);
				if ($shipment['extra_cover'] > 500 && $shipment['signature_on_delivery'] === false) {
					$priceItem = $this->addSignatureOnDelivery($priceItem, $shipment, true);
				}
			}

			$priceRequest->addItem( $priceItem );
		}

		$priceRequest->setFromState($from->getState())
		             ->setFromSuburb($from->getSuburb())
		             ->setToState($to->getState())
		             ->setToSuburb($to->getSuburb());
		$shipmentPricesRequest = new Shipment_Price_Request();
		foreach ($priceRequest->getItems() as $item) {
			$shipment = new Price_Request();
			$shipment
				->setFromPostcode($from->getPostcode())
				->setFromCountry($from->getCountry())
				->setToPostcode($to->getPostcode())
				->setToCountry($to->getCountry())
				->setFromState($from->getState())
				->setFromSuburb($from->getSuburb())
				->setToState($to->getState())
				->setToSuburb($to->getSuburb())
				->addItem($item);
			$shipmentPricesRequest->addShipment($shipment);
		}

		$prices = (new Shipment_Price(Utilities::get_api_connector(), $shipmentPricesRequest))->getItemPrices();

		$finalPrices = [];
		foreach ($prices as $price) {
			$finalPrices[] = [
				'calculated_price' => $price['shipment_summary']['total_cost'],
				'product_type' => Utilities::get_product_name($price['items'][0]['product_id']),
			];
		}

		return $finalPrices;
	}

	public function action() {
		return 'calculate_prices_with_surcharges';
	}

}
